// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICORE_NETWORK_UTILS_H_INCLUDED
#define YAMICORE_NETWORK_UTILS_H_INCLUDED

#include <yami4-core/core.h>
#include "details-fwd.h"

// selected per platform
#include <yami4-sys/details-types.h>

namespace yami
{

namespace details
{

struct ip_address
{
    host_type host;
    port_type port;
};

// parses addresses of any of the form:
// - "port"
// - "*:port"
// - "xxx.xxx.xxx.xxx:port"
// - "hostname:port"
core::result parse_address(
    core::allocator & alloc, const char * address, ip_address & ipa,
    core::io_error_function io_error_callback, void * io_error_callback_hint);

core::result get_host_by_name(const char * hostname, host_type & host,
    core::io_error_function io_error_callback, void * io_error_callback_hint);
port_type get_network_port(long port);

core::result set_nonblocking(io_descriptor_type & fd);
core::result set_reuseaddr(io_descriptor_type & fd);
core::result set_nodelay(io_descriptor_type & fd);
core::result set_keepalive(io_descriptor_type & fd);

} // namespace details

} // namespace yami

#endif // YAMICORE_NETWORK_UTILS_H_INCLUDED
