// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICORE_OPTIONS_H_INCLUDED
#define YAMICORE_OPTIONS_H_INCLUDED

#include <yami4-core/core.h>

namespace yami
{

namespace core
{
class parameters;
} // namespace core

namespace details
{

#ifdef YAMI4_WITH_OPEN_SSL
const std::size_t max_file_path_len = 200;
#endif // YAMI4_WITH_OPEN_SSL

struct options
{
    void init(const core::parameters * params);

    void override_with_user_values(const core::parameters * params);

    int tcp_listen_backlog;
    bool tcp_reuseaddr;
    bool tcp_nonblocking;
    std::size_t tcp_connect_timeout;
    bool tcp_nodelay;
    bool tcp_keepalive;
    std::size_t tcp_frame_size;

    std::size_t udp_frame_size;
    
    // POSIX and Windows:
    
    int unix_listen_backlog;
    bool unix_nonblocking;
    std::size_t unix_frame_size;

    bool file_nonblocking;
    std::size_t file_frame_size;

#ifdef YAMI4_WITH_QNX
    std::size_t qnx_frame_size;
#endif // YAMI4_WITH_QNX

    std::size_t max_input_frame_size;

#ifdef YAMI4_WITH_OPEN_SSL
    char ssl_certificate_file[max_file_path_len];
    char ssl_private_key_file[max_file_path_len];
#endif // YAMI4_WITH_OPEN_SSL
};

} // namespace details

} // namespace yami

#endif // YAMICORE_OPTIONS_H_INCLUDED
