// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include <yami4-core/parameter_iterator.h>
#include <yami4-core/allocator.h>
#include "parameters-details.h"
#include <cstring>

using namespace yami;
using namespace yami::core;

bool parameter_iterator::has_next() const
{
    return details::find_next_used(data_, num_of_entries_, current_index_) !=
        num_of_entries_;
}

void parameter_iterator::move_next()
{
    const std::size_t next_used =
        details::find_next_used(data_, num_of_entries_, current_index_);
    if (next_used != num_of_entries_)
    {
        current_index_ = next_used;
    }
}

parameter_entry parameter_iterator::current() const
{
    parameter_entry e;
    e.e_ = data_ + current_index_;
    return e;
}

void parameter_iterator::remove()
{
    details::entry & e = data_[current_index_];
    e.clear_name(*allocator_);
    e.clear_item(*allocator_);
    e.type = unused;
}
