// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICORE_DETAILS_TYPES_H_INCLUDED
#define YAMICORE_DETAILS_TYPES_H_INCLUDED

#include <pthread.h>
#include <sys/neutrino.h>

#include <cstdint>

namespace yami
{

namespace details
{

typedef int host_type;
typedef int port_type;

const int ip_any_address = 0;

typedef int io_descriptor_type;
const io_descriptor_type empty_io_descriptor = -1;

// Message receiver is a separate thread that is created
// for each QNX listener, its role is to block on MsgReceive
// and interrupt the selector when the new data packet arrives.
class channel_group;
class msg_receiver
{
public:
    bool start(channel_group * group, char * buf, std::size_t buf_size, pid_t pid, int chid);
    
    void stop();
    
    bool is_frame_ready() const;
    void received_details(bool & io_error_occurred,
        std::size_t & msg_size, std::uint32_t & sender_node, pid_t & sender_pid) const;
    void release_buffer();
    
    // driven by separate thread
    void do_run();
    
private:
    channel_group * group_;
    
    bool active_;
    char * buf_;
    std::size_t buf_size_;
    _msg_info message_info_;
    bool frame_buffer_busy_;
    bool io_error_occurred_;
    
    // "address" of the listener for waking it up
    pid_t pid_;
    int chid_;
    
    mutable pthread_mutex_t mtx_;
    pthread_cond_t cond_;
    
    pthread_t receiver_thread_;
};

} // namespace details

} // namespace yami

#endif // YAMICORE_DETAILS_TYPES_H_INCLUDED
