// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include "../../io_error_handler.h"

#include <cstdio>
#include <cstring>

#include <errno.h>

void yami::details::handle_io_error(
    const char * prefix,
    core::io_error_function io_error_callback,
    void * io_error_callback_hint)
{
    handle_io_error_with_code(prefix, errno,
        io_error_callback, io_error_callback_hint);
}

void yami::details::handle_io_error_with_code(
    const char * prefix,
    int error_code,
    core::io_error_function io_error_callback,
    void * io_error_callback_hint)
{
    if (io_error_callback != NULL)
    {
        const std::size_t max_msg_size = 500;
        char msg[max_msg_size];

        const char * error_str = std::strerror(error_code);
        std::snprintf(msg, max_msg_size, "%s: %s", prefix, error_str);

        try
        {
            io_error_callback(io_error_callback_hint, error_code, msg);
        }
        catch (...)
        {
            // ignore errors from user callbacks
        }
    }
}
