// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include <yami4-cpp/agent.h>
#include "agent_impl.h"
#include <yami4-cpp/value_publisher.h>

using namespace yami;

agent::agent(const parameters & options)
{
    pimpl_ = new details::agent_impl(options);
    pimpl_base_ = pimpl_;
}

agent::agent(event_callback & event_listener,
    const parameters & options)
{
    pimpl_ = new details::agent_impl(options, event_listener);
    pimpl_base_ = pimpl_;
}

agent::~agent()
{
    delete pimpl_;
}

std::string agent::add_listener(const std::string & listener)
{
    return pimpl_->add_listener(listener);
}

void agent::remove_listener(const std::string & listener)
{
    return pimpl_->remove_listener(listener);
}

void agent::register_raw_object(const std::string & object_name,
    void (* callback)(incoming_message & im, void * hint), void * hint)
{
    std::unique_ptr<details::incoming_message_dispatcher_base> object(
        new details::incoming_message_raw_dispatcher(callback, hint));

    pimpl_base_->register_object(object_name, object);
}

void agent::register_value_publisher(const std::string & object_name,
    value_publisher & publisher)
{
    publisher.register_at(*this, object_name);
}

void agent::unregister_object(const std::string & object_name)
{
    pimpl_->unregister_object(object_name);
}

void agent::open_connection(const std::string & target)
{
    pimpl_->open_connection(target);
}

void agent::open_connection(const std::string & target,
    const parameters & options)
{
    pimpl_->open_connection(target, options);
}

agent::outgoing_message_id agent::get_next_message_id()
{
    return pimpl_->get_next_message_id();
}

std::unique_ptr<outgoing_message> agent::send(
    const std::string & target,
    const std::string & object_name,
    const std::string & message_name,
    const serializable & content,
    std::size_t priority,
    bool auto_connect)
{
    return pimpl_->send(
        target, object_name, message_name, content, priority, auto_connect);
}

void agent::send(
    outgoing_message & message,
    const std::string & target,
    const std::string & object_name,
    const std::string & message_name,
    const serializable & content,
    std::size_t priority,
    bool auto_connect)
{
    pimpl_->send(message,
        target, object_name, message_name, content, priority, auto_connect);
}

std::unique_ptr<outgoing_message> agent::send(
    const std::string & target,
    const serializable & header,
    const serializable & content,
    outgoing_message_id message_id,
    std::size_t priority,
    bool auto_connect)
{
    return pimpl_->send(target, header, content, message_id,
        priority, auto_connect);
}

void agent::send(
    outgoing_message & message,
    const std::string & target,
    const serializable & header,
    const serializable & content,
    outgoing_message_id message_id,
    std::size_t priority,
    bool auto_connect)
{
    pimpl_->send(message, target, header, content, message_id,
        priority, auto_connect);
}

void agent::clean_outgoing_message_callback(outgoing_message_id id)
{
    pimpl_->clean_outgoing_message_callback(id);
}

void agent::send_one_way(const std::string & target,
    const std::string & object_name,
    const std::string & message_name,
    const serializable & content,
    std::size_t priority,
    bool auto_connect)
{
    pimpl_->send_one_way(
        target, object_name, message_name, content, priority, auto_connect);
}

void agent::send_one_way(const std::string & target,
    const serializable & header,
    const serializable & content,
    outgoing_message_id message_id,
    std::size_t priority,
    bool auto_connect)
{
    pimpl_->send_one_way(
        target, header, content, message_id, priority, auto_connect);
}

void agent::close_connection(const std::string & target, std::size_t priority)
{
    pimpl_->close_connection(target, priority);
}

void agent::hard_close_connection(const std::string & target)
{
    pimpl_->hard_close_connection(target);
}

void agent::get_outgoing_flow_state(std::size_t & current_level,
    std::size_t & high_water_mark, std::size_t & low_water_mark) const
{
    pimpl_->get_outgoing_flow_state(
        current_level, high_water_mark, low_water_mark);
}

void agent::get_channel_usage(std::size_t & max_allowed, std::size_t & used)
{
    pimpl_->get_channel_usage(max_allowed, used);
}

std::size_t agent::get_pending_outgoing_bytes(const std::string & target)
{
    return pimpl_->get_pending_outgoing_bytes(target);
}

void agent::run_worker()
{
    pimpl_->do_event_loop();
}

void agent::run_dispatcher(std::size_t dispatcher_index)
{
    pimpl_->do_message_dispatching(dispatcher_index);
}
