// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include <yami4-cpp/yami.h>

#include <cstdlib>
#include <iostream>
#include <string>

int main(int argc, char * argv[])
{
    if (argc != 2 && argc != 5)
    {
        std::cout << "need 1 or 4 parameters:\n"
            "   - server address\n"
            "   - outgoing high water mark\n"
            "   - outgoing low water mark\n"
            "   - number of iterations\n"
            "If only server address is given,"
            " the limits will have default values"
            " and the loop will be infinite\n";

        return EXIT_FAILURE;
    }

    const std::string server_address = argv[1];
    int num_of_iterations = -1;

    yami::parameters options;
    if (argc == 5)
    {
        int outgoing_high_water_mark = std::stoi(argv[2]);
        int outgoing_low_water_mark = std::stoi(argv[3]);
        num_of_iterations = std::stoi(argv[4]);

        options.set_integer("outgoing_high_water_mark",
            outgoing_high_water_mark);
        options.set_integer("outgoing_low_water_mark",
            outgoing_low_water_mark);
    }

    try
    {
        yami::agent client_agent(options);

        yami::parameters params;

        int index = 1;
        while (true)
        {
            params.set_integer("index", index);

            client_agent.send_one_way(server_address,
                "object", "message", params);

            std::cout
                << "posted message " << index << std::endl;

            if (num_of_iterations > 0)
            {
                if (index == num_of_iterations)
                {
                    break;
                }
            }

            ++index;
        }
    }
    catch (const std::exception & e)
    {
        std::cout << "error: " << e.what() << std::endl;
    }
}
