// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include "incoming_message_queue.h"

using namespace yami;
using namespace details;

incoming_message_queue::incoming_message_queue()
    : terminated_(false)
{
}

incoming_message_queue::~incoming_message_queue()
{
    while (queue_.empty() == false)
    {
        incoming_message_info * info = queue_.front();
        queue_.pop_front();

        if (info->body != NULL)
        {
            delete info->body;
        }

        delete info;
    }
}

void incoming_message_queue::push(std::unique_ptr<incoming_message_info> & info)
{
    {
        std::lock_guard<std::mutex> lock(mtx_);

        if (terminated_ == false)
        {
            if (info.get() != NULL)
            {
                queue_.push_back(info.release());
            }
            else
            {
                terminated_ = true;
            }
        }
    }
    cond_.notify_all();
}

std::unique_ptr<incoming_message_info> incoming_message_queue::pop()
{
    std::unique_ptr<incoming_message_info> info;
    {
        std::unique_lock<std::mutex> lock(mtx_);

        while (terminated_ == false && queue_.empty())
        {
            cond_.wait(lock);
        }
        
        // return NULL if the queue is terminated
        if (terminated_ == false)
        {
            info.reset(queue_.front());
            queue_.pop_front();
        }
    }

    return info;
}
