// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICPP_INCOMING_MESSAGE_QUEUE_H_INCLUDED
#define YAMICPP_INCOMING_MESSAGE_QUEUE_H_INCLUDED

#include "incoming_message_info.h"
#include <deque>
#include <memory>

#include <condition_variable>
#include <mutex>

namespace yami
{

namespace details
{

class incoming_message_queue
{
public:
    incoming_message_queue();
    ~incoming_message_queue();

    void push(std::unique_ptr<incoming_message_info> & incoming);

    std::unique_ptr<incoming_message_info> pop();

private:
    incoming_message_queue(const incoming_message_queue &);
    void operator=(const incoming_message_queue &);

    typedef std::deque<incoming_message_info *> queue_type;

    queue_type queue_;
    bool terminated_;
    std::condition_variable cond_;
    std::mutex mtx_;
};

} // namespace details

} // namespace yami

#endif // YAMICPP_INCOMING_MESSAGE_QUEUE_H_INCLUDED
