// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include "name_resolver.h"

using namespace yami;
using namespace details;

name_resolver::name_resolver()
    : any_callback_(NULL)
{
}

name_resolver::~name_resolver()
{
    for (auto & e : map_)
    {
        delete e.second;
    }

    if (any_callback_ != NULL)
    {
        delete any_callback_;
    }
}

void name_resolver::register_object(const std::string & name,
    std::unique_ptr<incoming_message_dispatcher_base> & object)
{
    std::lock_guard<std::mutex> lock(mtx_);

    if (name == "*")
    {
        any_callback_ = object.release();
    }
    else
    {
        const map_type::iterator it = map_.find(name);
        if (it != map_.end())
        {
            // replace existing registration

            delete it->second;

            it->second = object.release();
        }
        else
        {
            // create new registration

            map_[name] = object.release();
        }
    }
}

void name_resolver::unregister_object(const std::string & name)
{
    std::lock_guard<std::mutex> lock(mtx_);

    if (name == "*")
    {
        any_callback_ = NULL;
    }
    else
    {
        const map_type::iterator it = map_.find(name);
        if (it != map_.end())
        {
            delete it->second;

            map_.erase(it);
        }
    }
}

incoming_message_dispatcher_base * name_resolver::resolve(
    const std::string & name) const
{
    std::lock_guard<std::mutex> lock(mtx_);

    const map_type::const_iterator it = map_.find(name);
    if (it != map_.end())
    {
        return it->second;
    }
    else
    {
        return any_callback_;
    }
}
