// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICPP_NAME_RESOLVER_H_INCLUDED
#define YAMICPP_NAME_RESOLVER_H_INCLUDED

#include <yami4-cpp/incoming_message_dispatcher_base.h>
#include <map>
#include <memory>
#include <string>

#include <mutex>

namespace yami
{

namespace details
{

class name_resolver
{
public:
    name_resolver();
    ~name_resolver();

    void register_object(const std::string & name,
        std::unique_ptr<incoming_message_dispatcher_base> & object);

    void unregister_object(const std::string & name);

    incoming_message_dispatcher_base * resolve(
        const std::string & name) const;

private:
    name_resolver(const name_resolver &);
    void operator=(const name_resolver &);

    typedef std::map<std::string, incoming_message_dispatcher_base *>
        map_type;

    map_type map_;
    incoming_message_dispatcher_base * any_callback_;
    mutable std::mutex mtx_;
};

} // namespace details

} // namespace yami

#endif // YAMICPP_NAME_RESOLVER_H_INCLUDED
