// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICPP_NUMBER_GENERATOR_H_INCLUDED
#define YAMICPP_NUMBER_GENERATOR_H_INCLUDED

#include <random>

#include <mutex>

namespace yami
{

namespace details
{

// thread-safe generator of id sequences and pseudo-random values
class number_generator
{
public:
    // for random-value generation
    typedef std::mt19937::result_type result_type;
    static constexpr result_type min() { return std::mt19937::min(); }
    static constexpr result_type max() { return std::mt19937::max(); }
    
    number_generator();

    long long get_next_id();
    
    // generate pseudo-random value
    unsigned int operator()();

private:
    long long current_id_;
    std::random_device rd_;
    std::mt19937 mt_;
    
    std::mutex mtx_;
};

} // namespace details

} // namespace yami

#endif // YAMICPP_NUMBER_GENERATOR_H_INCLUDED
