// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include <yami4-cpp/outgoing_message.h>
#include "outgoing_message_info.h"
#include <yami4-cpp/outgoing_message_dispatcher_base.h>

using namespace yami;
using namespace details;

void outgoing_message_info::process_callback()
{
    if (message_callback != NULL && allow_callbacks)
    {
        // this function is called with mutex unlocked
        
        try
        {
            message_callback->dispatch(*out_msg);
        }
        catch (...)
        {
            // ignore user exceptions
        }
    }
}

void outgoing_message_info::bump_ref_count()
{
    ++ref_count;
}

void outgoing_message_info::dec_ref_count(bool stop_further_callbacks)
{
    std::size_t new_ref_count;
    {
        std::lock_guard<std::mutex> lock(mtx);
        
        new_ref_count = --ref_count;
        
        if (stop_further_callbacks)
        {
            allow_callbacks = false;
        }
    }

    if (new_ref_count == 0)
    {
        delete core_reply_header;
        delete header_wrapper;
        delete reply_body;
        delete reply_raw_buffer;
        delete out_msg;
        delete message_callback;
        
        delete this;
    }
}
