// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICPP_OUTGOING_MESSAGE_INFO_H_INCLUDED
#define YAMICPP_OUTGOING_MESSAGE_INFO_H_INCLUDED

#include <yami4-cpp/message_state.h>
#include <yami4-cpp/parameters.h>
#include <cstddef>
#include <vector>

#include <condition_variable>
#include <mutex>

namespace yami
{

class outgoing_message;

namespace details
{

class agent_impl;
class outgoing_message_dispatcher_base;

struct outgoing_message_info
{
    long long message_id;
    message_state state;
    std::size_t sent_bytes;
    std::size_t total_byte_count;

    std::mutex mtx;
    std::condition_variable cond;

    core::parameters * core_reply_header;
    mutable parameters * header_wrapper; // lazy initialized

    parameters * reply_body;
    std::vector<char> * reply_raw_buffer;
    std::string exception_msg;

    agent_impl * agent;
    
    // for callback (if used), owned by this info object
    outgoing_message * out_msg;
    outgoing_message_dispatcher_base * message_callback;
    bool allow_callbacks;

    // this is always called with mtx unlocked
    // and with bumped ref_count
    void process_callback();

    std::size_t ref_count;

    // this is always called with mtx locked
    // (or when it does not matter)
    void bump_ref_count();
    
    // this is always called with mtx unlocked
    void dec_ref_count(bool stop_further_callbacks = false);
};

} // namespace details

} // namespace yami

#endif // YAMICPP_OUTGOING_MESSAGE_INFO_H_INCLUDED
