// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICPP_OUTGOING_MESSAGE_MANAGER_H_INCLUDED
#define YAMICPP_OUTGOING_MESSAGE_MANAGER_H_INCLUDED

#include <yami4-cpp/parameters.h>
#include <map>
#include <memory>
#include <vector>

#include <mutex>

namespace yami
{

namespace details
{

struct outgoing_message_info;

class outgoing_message_manager
{
public:
    outgoing_message_manager() {}
    ~outgoing_message_manager();

    void put(long long message_id, outgoing_message_info * outgoing);

    void remove(long long message_id);

    void report_replied(long long message_id,
        std::unique_ptr<core::parameters> & header, std::unique_ptr<parameters> & body);
    void report_replied(long long message_id,
        std::unique_ptr<core::parameters> & header, std::unique_ptr<std::vector<char> > & raw_buffer);

    void report_rejected(long long message_id,
        std::unique_ptr<core::parameters> & header, const std::string & reason);

private:
    outgoing_message_manager(const outgoing_message_manager &);
    void operator=(const outgoing_message_manager &);
    
    void report(long long message_id, std::unique_ptr<core::parameters> & header,
        std::unique_ptr<parameters> * body,                // only one
        std::unique_ptr<std::vector<char> > * raw_buffer,  // of these three
        const std::string * reason);                       // is not-NULL

    typedef std::map<long long, outgoing_message_info *> map_type;

    void do_remove(const map_type::iterator & it);
    
    map_type map_;
    std::mutex mtx_;
};

} // namespace details

} // namespace yami

#endif // YAMICPP_OUTGOING_MESSAGE_MANAGER_H_INCLUDED
