// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include "water_flow_manager.h"
#include <yami4-cpp/errors.h>

using namespace yami;
using namespace details;

water_flow_manager::water_flow_manager()
    : current_level_(0)
{
}

void water_flow_manager::set_limits(
    std::size_t high_mark, std::size_t low_mark)
{
    if (high_mark < low_mark)
    {
        throw yami::yami_logic_error("Wrong limit values.");
    }

    high_water_mark_ = high_mark;
    low_water_mark_ = low_mark;
}

water_flow_control water_flow_manager::increase()
{
    water_flow_control result = no_change;

    std::lock_guard<std::mutex> lock(mtx_);

    ++current_level_;
    if (current_level_ >= high_water_mark_)
    {
        result = suppress;
    }

    return result;
}

water_flow_control water_flow_manager::decrease()
{
    water_flow_control result = no_change;

    std::lock_guard<std::mutex> lock(mtx_);

    --current_level_;
    if (current_level_ < low_water_mark_)
    {
        result = allow;
    }

    return result;
}

void water_flow_manager::get_state(std::size_t & current_level,
    std::size_t & high_water_mark, std::size_t & low_water_mark) const
{
    std::lock_guard<std::mutex> lock(mtx_);

    current_level = current_level_;
    high_water_mark = high_water_mark_;
    low_water_mark = low_water_mark_;
}
