// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMICPP_WATER_FLOW_MANAGER_H_INCLUDED
#define YAMICPP_WATER_FLOW_MANAGER_H_INCLUDED

#include <cstddef>

#include <mutex>

namespace yami
{

namespace details
{

enum water_flow_control { suppress, allow, no_change };

class water_flow_manager
{
public:
    water_flow_manager();

    void set_limits(std::size_t high_mark, std::size_t low_mark);

    water_flow_control increase();
    water_flow_control decrease();

    void get_state(std::size_t & current_level,
        std::size_t & high_water_mark,
        std::size_t & low_water_mark) const;

private:
    std::size_t current_level_;
    std::size_t high_water_mark_;
    std::size_t low_water_mark_;
    mutable std::mutex mtx_;
};

} // namespace details

} // namespace yami

#endif // YAMICPP_WATER_FLOW_MANAGER_H_INCLUDED
