# Copyright Maciej Sobczak 2008-2022.
# This file is part of YAMI4.
# See the package-level LICENSE.txt file.

import sys
import yami

if len(sys.argv) != 4 and len(sys.argv) != 5:
    print("expecting three or four parameters:",
          "server destination int_1 int_2 [mode]")
    exit()

server_address = sys.argv[1]

try:
    a = int(sys.argv[2])
    b = int(sys.argv[3])
except ValueError:
    print("cannot parse the second or third parameter")
    exit()

if len(sys.argv) == 5:
    mode = sys.argv[4]
else:
    mode = None

try:
    with yami.Agent() as client_agent:

        # create custom header with optional mode field:
        message_id = client_agent.get_next_message_id()
        header = {"type":"message",
                  "message_id":yami.Long(message_id),
                  "object_name":"calculator",
                  "message_name":"calculate"}
        if mode != None:
            header["mode"] = mode

        params = {"a":a, "b":b}

        with client_agent.send_with_header(
            server_address, header, params, message_id) as msg:

            msg.wait_for_completion()
            state = msg.get_state()
            if state[0] == yami.OutgoingMessage.REPLIED:
                reply = msg.get_reply()

                sum = reply["sum"]
                difference = reply["difference"]
                product = reply["product"]

                if "ratio" in reply:
                    ratio = reply["ratio"]
                    ratio_defined = True
                else:
                    ratio_defined = False

                print("sum        =", sum)
                print("difference =", difference)
                print("product    =", product)

                print("ratio      = ", end="")
                if ratio_defined:
                    print(ratio)
                else:
                    print("<undefined>")

            elif state[0] == yami.OutgoingMessage.REJECTED:
                print("The message has been rejected:",
                      msg.get_exception_msg())
            else:
                print("The message has been abandoned.")
            
            # check custom header in reply
            reply_header = msg.get_header()
            if "mode_received" in reply_header:
                if reply_header["mode_received"]:
                    print("the server has received mode setting")

except Exception as e:
    print("error:", e)
