# Copyright Maciej Sobczak 2008-2022.
# This file is part of YAMI4.
# See the package-level LICENSE.txt file.

import sys
import yami

if len(sys.argv) != 2:
    print("expecting one parameter: server destination")
    exit()

server_address = sys.argv[1]
agent_object = None

def call(msg):

    # extract the custom header field named "mode", if it exists

    header = msg.get_header()
    mode = None
    if "mode" in header:
        mode = header["mode"]

    # extract the parameters for calculations

    params = msg.get_parameters()

    a = params["a"]
    b = params["b"]

    # prepare the answer with results of four calculations

    reply_params = {"sum":a+b, "difference":a-b, "product":a*b}

    # if the ratio cannot be computed,
    # it is not included in the response
    # the client will interpret that fact properly
    if b != 0:
        reply_params["ratio"] = int(a / b)

    # server can reply in a basic way:
    #
    # msg.reply(reply_params)
    #
    # but it can also use customized header fields to create a message
    # that acts as a first-class response:

    message_id = msg.get_message_id()
    reply_header = {"type":"reply",
                    "message_id":yami.Long(message_id),
                    "mode_received":(mode != None)}
    agent_object.send_one_way_with_header(msg.get_source(), reply_header, reply_params, message_id)

    if mode == "hex":
        print("got message with parameters", str.format("0x{:x}", a),
              "and", str.format("0x{:x}", b))
    else:
        print("got message with parameters", a, "and", b)
    print("response has been sent back")

try:
    with yami.Agent() as server_agent:
        agent_object = server_agent # so that message callback can use it

        resolved_address = server_agent.add_listener(server_address)

        print("The server is listening on", resolved_address)

        server_agent.register_object("calculator", call)

        # block
        dummy = sys.stdin.read()

except Exception as e:
    print("error:", e)
