// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMI4BROKER_ROUTING_H
#define YAMI4BROKER_ROUTING_H

#include "broker.h"

#include <yami4-core/parameters.h>
#include <yami4-core/serializable.h>

#include <string>

namespace broker
{

void init_routing(std::size_t max_subscriptions,
    std::size_t max_client_queue,
    overflow_policy subscription_overflow_policy);

void subscribe(
    const std::string & tags, const std::string & target_object,
    const std::string & target_location);

void revoke_subscription(
    const std::string & tags, const std::string & target_object,
    const std::string & target_location);

void set_forwarding(
    const std::string & tags, const std::string & target_location);

void iterate_matching_subscriptions(
    const std::string & tags,
    yami::core::parameters & header,
    const yami::core::serializable & body,
    bool (*process)(
        const std::string & target_object,
        const std::string & target_location,
        const std::string & tags,
        yami::core::parameters & header,
        const yami::core::serializable & body,
        yami::core::message_progress_function progress_handler,
        void * progress_hint),
    bool & out_overflow);

yami::core::result fill_detailed_stats(yami::core::parameters & params);

} // namespace broker

#endif // YAMI4BROKER_ROUTING_H
