// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include <yami4-cpp/yami.h>

#include <cstdlib>
#include <iostream>

#include <chrono>
#include <thread>

int main(int argc, char * argv[])
{
    if (argc != 5)
    {
        std::cout <<
            "expecting 4 parameters:\n"
            "    broker address\n"
            "    list of tags for publishing\n"
            "    message text\n"
            "    delay in ms\n";

        return EXIT_FAILURE;
    }

    const std::string & broker_address = argv[1];
    const std::string & list_of_tags = argv[2];
    const std::string & text = argv[3];

    const int wait_time = std::stoi(argv[4], NULL);

    try
    {
        yami::agent publisher_agent;

        yami::parameters content;

        content.set_string("value", text);

        int counter = 1;
        while (true)
        {
            publisher_agent.send_one_way(
                broker_address, list_of_tags, "publish", content);

            std::cout << "sent " << counter << std::endl;
            ++counter;

            std::this_thread::sleep_for(std::chrono::milliseconds(wait_time));
        }
    }
    catch (const std::exception & e)
    {
        std::cout << "error: " << e.what() << '\n';
    }
}
