// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include "log.h"

#include <ctime>
#include <cstdio>

using namespace logger;

namespace // unnamed
{

std::FILE * out_file_ = stdout;
std::string base_file_name_;
std::size_t current_size_ = 0;
std::size_t max_size_ = 0;
bool line_sync_ = false;

bool activity_flags_[5] = { true, true, true, true, true };

char format_buf_[20];

const char * current_time_formatted()
{
    std::time_t time = std::time(NULL);
    std::tm * t = std::localtime(&time);

    std::sprintf(format_buf_, "%d-%02d-%02d %02d:%02d:%02d",
        t->tm_year + 1900,
        t->tm_mon + 1,
        t->tm_mday,
        t->tm_hour,
        t->tm_min,
        t->tm_sec);

    return format_buf_;
}

std::string new_log_file_name()
{
    if (max_size_ != 0)
    {
        std::time_t time = std::time(NULL);
        std::tm * t = std::localtime(&time);

        std::sprintf(format_buf_, "%d%02d%02d%02d%02d%02d",
            t->tm_year + 1900,
            t->tm_mon + 1,
            t->tm_mday,
            t->tm_hour,
            t->tm_min,
            t->tm_sec);
        
        return base_file_name_ + '.' + format_buf_;
    }
    else
    {
        return base_file_name_;
    }
}

} // unnamed namespace

const char * logger::module_name(module m)
{
    switch (m)
    {
    case main:
        return "MAIN";
    case subscriptions:
        return "SUBSCRIPTIONS";
    case queues:
        return "QUEUES";
    case store:
        return "STORE";
    case messages:
        return "MESSAGES";
    default:
        return "";
    }
}

void logger::put(module m, const std::string & msg)
{
    if (activity_flags_[m])
    {
        if ((out_file_ == NULL) ||
            ((max_size_ > 0) && (current_size_ >= max_size_)))
        {
            if (out_file_ != NULL)
            {
                std::fclose(out_file_);
            }
            
            out_file_ = std::fopen(new_log_file_name().c_str(), "w");
            current_size_ = 0;
        }
        
        std::fprintf(out_file_,
            "%s [%s] %s\n",
            current_time_formatted(), module_name(m), msg.c_str());
        
        if (line_sync_)
        {
            std::fflush(out_file_);
        }
        
        ++current_size_;
    }
}

void logger::set_output_file(
    const std::string & file_name,
    std::size_t max_size, bool line_sync)
{
    if (file_name != "")
    {
        base_file_name_ = file_name;
        max_size_ = max_size;
        line_sync_ = line_sync;
        out_file_ = NULL;
    }
}

void logger::enable(module m, bool active)
{
    activity_flags_[m] = active;
}
