// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMI4SERVICES_LOG_H
#define YAMI4SERVICES_LOG_H

#include <string>

namespace logger
{

enum module { main, subscriptions, queues, store, messages, last_module_ };

// Returns string representation of the module name.
const char * module_name(module m);

// Writes message associated with the given module to the log file.
void put(module m, const std::string & msg);

// Sets the name of output file.
// If not set or empty, the log is printed on standard output.
// If max_size > 0, the log file name has a time stamp and
// new file is created after reaching the given limit.
// If line_sync, the log file is synchronized after each line.
void set_output_file(const std::string & file_name,
    std::size_t max_size, bool line_sync);

// Enables or disables output for the given module.
// By default all modules are enabled.
void enable(module m, bool active);

} // namespace logger

#endif // YAMI4SERVICES_LOG_H
