// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMI4SERVICES_PROPERTIES_H
#define YAMI4SERVICES_PROPERTIES_H

#include <yami4-core/parameters.h>

#include <string>

namespace properties
{

// Loads the properties from a given file and from command line options.
// The new mappings (name=value) will be applied onto the existing ones.
void load_properties(const char * config_file_name, int argc, char * argv[]);

// Clears the properties that were read from file(s).
// Does not clear the properties set by command-line options.
void clear();

// Returns True is the given property is defined either in
// the regular property set or in the properties defined by
// command-line options.
bool is_defined(const std::string & name);

// Returns the value for the given name, or Default if no
// such name is found.
// During the lookup, the command-line properties override those
// read from file(s).
std::string get(const std::string & name, const std::string & default_value = "");

// Fill the parameters object with agent options as for the core module.
void core_agent_options(yami::core::parameters & options);

} // namespace properties

#endif // YAMI4SERVICES_PROPERTIES_H
