// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include "utils.h"

#include <cctype>

using namespace utils;

std::string utils::trim(const std::string & s)
{
    const char * b = s.c_str();
    const char * e = b + s.size() - 1;
    
    while (std::isspace(*b)) { ++b; };
    while (e > b && std::isspace(*e)) { --e; };
    
    return std::string(b, e - b + 1);
}

std::string utils::to_lower(const std::string & s)
{
    std::string res(s);
    
    for (char & c : res)
    {
        c = std::tolower(c);
    }
    
    return res;
}

yami::core::result utils::get_string(
    const yami::core::parameters & params, const char * field_name,
    std::string & out_result)
{
    const char * value;
    std::size_t value_len;

    yami::core::result res =
        params.get_string(field_name, value, value_len);
    if (res == yami::core::ok)
    {
        out_result.assign(value, value_len);
    }

    return res;
}

std::string utils::result_to_string(yami::core::result res)
{
    switch (res)
    {
    case yami::core::no_such_name:
        return "No such name.";
    case yami::core::bad_type:
        return "Bad type.";
    case yami::core::no_such_index:
        return "No such index.";
    case yami::core::no_memory:
        return "Not enough memory.";
    case yami::core::nesting_too_deep:
        return "Nesting of parameters is too deep.";
    case yami::core::not_enough_space:
        return 
            "Not enough space or not enough data in the buffer.";
    case yami::core::no_entries:
        return "No entries found.";
    case yami::core::unexpected_value:
        return 
            "The value that was given or received is incorrect.";
    case yami::core::bad_protocol:
        return "The given protocol is not supported.";
    case yami::core::io_error:
        return "I/O error.";
    case yami::core::timed_out:
        return "Operation timed out.";
    case yami::core::channel_closed:
        return "The channel was closed.";
    case yami::core::bad_state:
        return "The given object is in the wrong state.";
    default:
        // operation completed successfully
        return "";
    }
}
