// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include "name_server-configuration.h"
#include "name_server-messaging.h"
#include "name_server-storage.h"
#include "../common/log.h"
#include "../common/utils.h"

#include <cstdio>

#include <chrono>
#include <thread>

int main(int argc, char * argv[])
{
    try
    {
        bool success = name_server::init_config(argc, argv);

        if (success)
        {
            logger::set_output_file(
                name_server::log_file_name(),
                name_server::log_file_max_size(),
                name_server::log_file_line_sync());
            
            for (logger::module m = logger::main; m != logger::last_module_;
                 m = static_cast<logger::module>(m + 1))
            {
                logger::enable(m, name_server::log_enabled(m));
            }

            logger::put(logger::main, "initialized configuration settings");

            const std::string & data_directory = name_server::data_directory();

            name_server::init_storage(data_directory);

            // take into account run-time options as defined in configuration
            yami::core::parameters options;
            name_server::agent_options(options);
            
            name_server::init_messaging(options,
                name_server::listener(),
                name_server::announce_file());
        }
        else
        {
            logger::put(logger::main, "configuration not initialized properly");
        }

        while (true)
        {
            std::this_thread::sleep_for(std::chrono::seconds(10)); // arbitary delay
        }
    }
    catch (const std::exception & e)
    {
        logger::put(logger::main, "unknown exception in main task:");
        logger::put(logger::main, e.what());
    }
}
