// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include "name_server-configuration.h"
#include "../common/properties.h"

#include <cstdio>

using namespace name_server;

namespace // unnamed
{

const char * default_config_file_name = "yami4names.cfg";

} // unnamed namespace

bool name_server::init_config(int argc, char * argv[])
{
    bool result = false;

    try
    {
        if (argc != 1)
        {
            properties::load_properties(argv[1], argc, argv);
        }
        else
        {
            properties::load_properties(default_config_file_name, argc, argv);
        }

        result = true;
    }
    catch (...)
    {
        // error while loading config file
    }

    return result;
}

std::string name_server::listener()
{
    return properties::get("listener", "tcp://*:*");
}

std::string name_server::announce_file()
{
    return properties::get("announce.file");
}

std::string name_server::data_directory()
{
    return properties::get("data-directory", "data");
}

std::string name_server::log_file_name()
{
    return properties::get("log_file_name");
}

std::size_t name_server::log_file_max_size()
{
    const std::string & s = properties::get("log_file_max_size", "0");

    int d;
    const int res = std::sscanf(s.c_str(), "%d", &d);

    if ((res == 1) && (d >= 0))
    {
        return static_cast<std::size_t>(d);
    }
    else
    {
        return 0;
    }
}

bool name_server::log_file_line_sync()
{
    const std::string & s = properties::get("log_file_line_sync", "true");
    
    return s == "true";
}

bool name_server::log_enabled(logger::module m)
{
    const std::string & key = std::string("log.") + logger::module_name(m);
    const std::string & value = properties::get(key, "false");

    return value == "true";
}

void name_server::agent_options(yami::core::parameters & options)
{
    properties::core_agent_options(options);
}
