// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#ifndef YAMI4NAMES_CONFIGURATION_H
#define YAMI4NAMES_CONFIGURATION_H

#include "../common/log.h"

#include <yami4-core/parameters.h>

#include <string>

namespace name_server
{

// Initializes all configuration options from the config file
// that is named in the first command-line parameter or
// from the yami4names.cfg file if no command-line argument is given.
bool init_config(int argc, char * argv[]);

// Returns the listener endpoint for the messaging part.
std::string listener();

// Returns the file name where the resolved listener target should be written.
std::string announce_file();

// Returns the path to the data directory.
std::string data_directory();

// Log file name. If not specified, log is printed on standard output.
std::string log_file_name();

// Max size, in entries of the log, after which a new file is created.
// If not specified, the log file grows indefinitely and does not have
// the timestamp attached to its name.
std::size_t log_file_max_size();

// Whether the log file should be synchronized after each line.
bool log_file_line_sync();

// Initial log level.
bool log_enabled(logger::module m);

// Agent options.
void agent_options(yami::core::parameters & options);

} // namespace name_server

#endif // YAMI4NAMES_CONFIGURATION_H
