// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include "name_server-storage.h"

#include "../common/log.h"
#include "../common/utils.h"

#include <cstdio>
#include <filesystem>

using namespace name_server;

namespace // unnamed
{

std::string data_directory_;

name_map map_;

const std::string empty_string;

const std::size_t max_binding_size = 200;

}

void name_server::init_storage(const std::string & data_directory)
{
    data_directory_ = data_directory;

    std::size_t num_of_entries = 0;

    std::filesystem::path dir_path(data_directory);

    if (std::filesystem::exists(dir_path) &&
        std::filesystem::is_directory(dir_path))
    {
        std::filesystem::directory_iterator it(dir_path);
        std::filesystem::directory_iterator end;

        for ( ; it != end; ++it)
        {
            std::filesystem::path entry_path(it->path());

            if (std::filesystem::is_regular_file(entry_path))
            {
                std::FILE * file = std::fopen(entry_path.string().c_str(), "r");

                char buf[max_binding_size];
                std::fgets(buf, max_binding_size, file);
                for (std::size_t i = 0; i != max_binding_size; ++i)
                {
                    if (buf[i] == '\n')
                    {
                        buf[i] = '\0';
                        break;
                    }
                }

                map_[entry_path.filename().string()] = buf;
                
                std::fclose(file);

                ++num_of_entries;
            }
        }
    }

    logger::put(logger::store, "initialized persistent store at " + data_directory);

    logger::put(logger::store, "recovered " +
        std::to_string(num_of_entries) + " bindings from store");
}

void name_server::store(const std::string & object_name, const std::string & location)
{
    std::filesystem::path dir_path(data_directory_);
    std::filesystem::path file_path(dir_path);
    file_path /= object_name;

    std::FILE * file = std::fopen(file_path.string().c_str(), "w");

    std::fputs(location.c_str(), file);
    std::fputs("\n", file);

    std::fclose(file);

    map_[object_name] = location;
}

const std::string & name_server::resolve(const std::string & object_name)
{
    name_map::const_iterator it = map_.find(object_name);
    if (it != map_.end())
    {
        return it->second;
    }
    else
    {
        return empty_string;
    }
}

const name_map & name_server::name_mapping()
{
    return map_;
}
