// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include <yami4-cpp/yami.h>

#include <cstdlib>
#include <iostream>

int main(int argc, char * argv[])
{
    if (argc != 4)
    {
        std::cout <<
            "expecting 3 parameters:\n"
            "    name server address\n"
            "    object name\n"
            "    object location\n";

        return EXIT_FAILURE;
    }

    const std::string & name_server_address = argv[1];
    const std::string & object_name = argv[2];
    const std::string & object_location = argv[3];

    try
    {
        yami::agent agent;

        // this step is needed on QNX, to ensure
        // that there exists a possibility to receive responses
        //(void)agent.add_listener("qnx://*");
        
        yami::parameters params;

        params.set_string("object", object_name);
        params.set_string("location", object_location);
        
        auto om = agent.send(name_server_address,
            "names", "bind", params);

        om->wait_for_completion();
        const yami::message_state state = om->get_state();
        if (state == yami::rejected)
        {
            std::cout << "bind rejected: "
                << om->get_exception_msg() << '\n';
        }
    }
    catch (const std::exception & e)
    {
        std::cout << "error: " << e.what() << '\n';
    }
}
