// Copyright Maciej Sobczak 2008-2022.
// This file is part of YAMI4.
// See the package-level LICENSE.txt file.

#include <yami4-cpp/yami.h>

#include <cstdlib>
#include <iostream>

int main(int argc, char * argv[])
{
    if (argc != 2)
    {
        std::cout <<
            "expecting 1 parameter:\n"
            "    name server address\n";

        return EXIT_FAILURE;
    }

    const std::string & name_server_address = argv[1];

    try
    {
        yami::agent agent;

        // this step is needed on QNX, to ensure
        // that there exists a possibility to receive responses
        //(void)agent.add_listener("qnx://*");
        
        auto om = agent.send(name_server_address, "names", "list");

        om->wait_for_completion();
        const yami::message_state state = om->get_state();
        if (state == yami::replied)
        {
            const yami::parameters & reply = om->get_reply();

            std::size_t length = reply.get_string_array_length("object");
            for (std::size_t i = 0; i != length; ++i)
            {
                const std::string & object_name =
                    reply.get_string_in_array("object", i);
                const std::string & object_location =
                    reply.get_string_in_array("location", i);

                std::cout << object_name << " " << object_location << '\n';
            }
        }
    }
    catch (const std::exception & e)
    {
        std::cout << "error: " << e.what() << '\n';
    }
}
